<?php

namespace App\Http\Controllers\API\V1;

use DB;
use App\Models\User;
use Illuminate\Support\Str;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;

class AuthController extends Controller
{
    function register(Request $request) {
        try {
            $data = $request->validate([
                "name" => "required",
                "email" => "required|email|unique:users,Email",
                "password" => "required",
            ]);
            $data['password'] = Hash::make($data['password']);
            $user = User::create($data);
            $token = $user->createToken($user->email);
            
            return response()->json(['status'=>'success','message'=>'user created','access_token'=> $token->plainTextToken,'user'=>$user],200);
        } catch (\Exception $e) {
            return response()->json(['status' => 'error','message' => $e->getMessage()]);
        }
        
    }

    function login(Request $request) {
        
        try {
            $request->validate([
                "email" => "required",
                "password" => "required",
            ]);
    
            $credentials = $request->only('email', 'password');
            
            if (Auth::attempt($credentials)) {
                $token = $request->user()->createToken($request->user()->email)->plainTextToken;
    
                return response()->json(['status'=>'success','message'=>'login successfully','access_token' => $token,'user'=>Auth::user()], 200);
            }
            return response()->json(['status'=>'unauthenticated','message' => 'Invalid credentials'], 401);
        } catch (\Exception $e) {
            return response()->json(['status' => 'error','message' => $e->getMessage()]);
        }
        
    }
}
