<?php

namespace App\Http\Controllers\master;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\UnitOfMeasure;
use App\Models\SystemParameter;
use App\Models\CommodityCompanyRelation;
use App\Models\CONT_CommodityUOM;
use Validator;
use DB;

class MeasureController extends Controller
{

    private function validateAccesToken($access_token)
    {
        return ['id' => 1, 'name' => 'ADMIN'];
    }

    public function datatable_list(REQUEST $request)
    {
        $status   = "0";
        $message  = "";
        $o_data   = [];
        $errors   = [];
        $validator = Validator::make($request->all(), [
            'access_token' => 'required'
        ]);

        if ($validator->fails()) {
            $status = "0";
            $message = "Validation error occured";
            $errors = $validator->messages();
        } else {
            $user    = $this->validateAccesToken($request->access_token);
            return $this->get_measure_list($request->all());
        }

        return response()->json(['status' => $status, 'error' => (object)$errors, 'message' => $message, 'oData' => (object)$o_data], 200);
    }

    public function get_measure_list($postData = null)
    {

        $response = array();



        ## Read value

        $draw = $postData['draw'];

        $start = $postData['start'];

        $rowperpage = $postData['length']; // Rows display per page

        $columnIndex = $postData['order'][0]['column']; // Column index

        $columnName = $postData['columns'][$columnIndex]['data']; // Column name

        $columnSortOrder = $postData['order'][0]['dir']; // asc or desc

        $searchValue = $postData['search']['value']; // Search value







        $filter = [];



        ## Search 

        $searchQuery = "";

        if ($searchValue != '') {

            $searchQuery = " (UOM_IdUnit like '%" . $searchValue . "%' or UOM_NmeUnit like '%" . $searchValue . "%' or UOM_IdBaseUnit like '%" . $searchValue . "%' or UOM_FlgPackingUnit like '%" . $searchValue . "%' or UOM_NmePackingType like '%" . $searchValue . "%'  ) ";
        }



        ## Total number of records without filtering

        $list = $this->_get_measure_list('', FALSE, $filter);
        $totalRecords = $list->get()->count();



        ## Total number of record with filtering

        $list = $this->_get_measure_list($searchQuery, FALSE, $filter);
        $totalRecordwithFilter = $list->get()->count();



        ## Fetch records

        $list = $this->_get_measure_list($searchQuery, FALSE, $filter);
        $list = $list->orderBy($columnName, $columnSortOrder);
        $list = $list->skip($start)->take($rowperpage);
        $records = $list->get();



        $data = array();



        foreach ($records as $record) {

            $action = '';





            $data[] = array(



                "UOM_IdUnit"               => $record->UOM_IdUnit,
                "UOM_NmeUnit"                => $record->UOM_NmeUnit,
                "UOM_IdBaseUnit"         => $record->UOM_IdBaseUnit,
                "UOM_FlgPackingUnit"         => $record->UOM_FlgPackingUnit,
                "UOM_NmePackingType"          => $record->UOM_NmePackingType,
                "UOM_ValNoOfUnits"                 => $record->UOM_ValNoOfUnits,
                "Action" => $action


            );
        }



        ## Response

        $response = array(

            "draw" => intval($draw),

            "iTotalRecords" => $totalRecords,

            "iTotalDisplayRecords" => $totalRecordwithFilter,

            "aaData" => $data

        );



        return $response;
    }

    private function _get_measure_list($searchQuery = '', $needSql = FALSE, $filter = [])
    {

        $list = UnitOfMeasure::select('*');
        if ($filter) {
            $list = $list->where($filter);
        }
        if ($searchQuery != '') {
            $list = $list->whereRaw($searchQuery);
        }
        if ($needSql) {
            return $list = $list->toSql();
        } else {
            return $list;
        }
    }

    public function store(REQUEST $request)
    {
        $status   = "0";
        $message  = "";
        $o_data   = [];
        $errors   = [];
        $validator = Validator::make($request->all(), [
            'UOM_IdUnit'           => 'required',
            'UOM_NmeUnit'           => 'required',
            'UOM_FlgWeightBasis'    => 'required',
            'UOM_FlgPackingUnit'    => 'required'
        ], [
            'UOM_IdUnit.required'           => 'this field is required',
            'UOM_NmeUnit.required'           => 'this field is required',
            'UOM_FlgWeightBasis.required'    => 'this field is required',
            'UOM_FlgPackingUnit.required'    => 'this field is required'
        ]);

        if ($validator->fails()) {
            $status = "0";
            $message = "Validation error occured";
            $errors = $validator->messages();
        } else {
            $user = $this->validateAccesToken($request->access_token);

            $check = UnitOfMeasure::where(['UOM_IdUnit' => strtoupper($request->UOM_IdUnit)])->get();
            if ($check->count() > 0) {
                $message = "Measure already exist";
            } else {
                DB::beginTransaction();
                try {

                    $data = [
                        "UOM_IdUnit" => str_replace(' ', '_', $request->UOM_IdUnit),
                        "UOM_NmeUnit" => str_replace(' ', '_', $request->UOM_NmeUnit),
                        "UOM_FlgWeightBasis" => $request->UOM_FlgWeightBasis,
                        "UOM_FlgPackingUnit" => $request->UOM_FlgPackingUnit,
                        "UOM_NmepackingType" => $request->UOM_NmePackingType,
                        "UOM_ValNoOfUnits" => $request->UOM_ValNoOfUnits,
                        "UOM_ValConversionFactor" => $request->UOM_ValConversionFactor,
                        "UOM_FlgPrintNoOfBags" => $request->UOM_FlgPrintNoOfBags,
                        "UOM_StatutoryUnitCode" => $request->UOM_StatutoryUnitCode,
                        "UOM_StatutoryUnitName" => $request->UOM_StatutoryUnitName,
                        "UOM_IdBaseUnit" => $request->UOM_IdUnit,
                        "UOM_DtLastUpdated" => date('Y-m-d H:i:s'),
                        "UOM_NmeUserLastUpdated" => 'Admin',
                        "UOM_PACKINGMATERIALWEIGHT1" => $request->UOM_PACKINGMATERIALWEIGHT1,
                        "UOM_PACKINGMATERIALWEIGHT2" => $request->UOM_PACKINGMATERIALWEIGHT2,
                        "UOM_PACKINGMATERIALWEIGHT3" => $request->UOM_PACKINGMATERIALWEIGHT3,
                        "UOM_PACKINGMATERIALWEIGHT4" => $request->UOM_PACKINGMATERIALWEIGHT4,
                        "UOM_PACKINGMATERIALWEIGHT5" => $request->UOM_PACKINGMATERIALWEIGHT5,
                        "UOM_PACKINGMATERIALWEIGHT6" => $request->UOM_PACKINGMATERIALWEIGHT6,
                        "UOM_PACKINGMATERIALWEIGHT7" => $request->UOM_PACKINGMATERIALWEIGHT7,
                        "UOM_BAGSPERSTDPALLET"       => $request->UOM_BAGSPERSTDPALLET,
                        "UOM_IndDisplayOrderBy"       => $request->UOM_IndDisplayOrderBy,
                    ];

                    $Measure = UnitOfMeasure::create($data);

                    DB::commit();
                    $status = "1";
                    $message = "Record saved successfully";
                } catch (\Exception $e) {
                    DB::rollback();
                    $message = "Transaction Faild: " . $e->getMessage();
                }
            }
        }

        return response()->json(['status' => $status, 'errors' => (object)$errors, 'message' => $message, 'oData' => (object)$o_data], 200);
    }

    function edit($id)
    {
        return UnitOfMeasure::where('UOM_IdUnit', $id)->first();
    }
    public function update(REQUEST $request)
    {
        $status   = "0";
        $message  = "";
        $o_data   = [];
        $errors   = [];
        $validator = Validator::make(
            $request->all(),
            [
                'UOM_IdUnit'           => 'required',
                'UOM_NmeUnit'           => 'required',
                'UOM_FlgWeightBasis'    => 'required',
                'UOM_FlgPackingUnit'    => 'required'
            ],
            [
                'UOM_IdUnit.required'           => 'this field is required',
                'UOM_NmeUnit.required'           => 'this field is required',
                'UOM_FlgWeightBasis.required'    => 'this field is required',
                'UOM_FlgPackingUnit.required'    => 'this field is required'
            ]
        );

        if ($validator->fails()) {
            $status = "0";
            $message = "Validation error occured";
            $errors = $validator->messages();
        } else {
            $user = $this->validateAccesToken($request->access_token);

            DB::beginTransaction();
            try {

                $data = [
                    "UOM_NmeUnit" => str_replace(' ', '_', $request->UOM_NmeUnit),
                    "UOM_FlgWeightBasis" => $request->UOM_FlgWeightBasis,
                    "UOM_FlgPackingUnit" => $request->UOM_FlgPackingUnit,
                    "UOM_NmepackingType" => $request->UOM_NmePackingType,
                    "UOM_ValNoOfUnits" => $request->UOM_ValNoOfUnits,
                    "UOM_ValConversionFactor" => $request->UOM_ValConversionFactor,
                    "UOM_FlgPrintNoOfBags" => $request->UOM_FlgPrintNoOfBags,
                    "UOM_StatutoryUnitCode" => $request->UOM_StatutoryUnitCode,
                    "UOM_StatutoryUnitName" => $request->UOM_StatutoryUnitName,
                    "UOM_IdBaseUnit" => $request->UOM_IdUnit,
                    "UOM_DtLastUpdated" => date('Y-m-d H:i:s'),
                    "UOM_NmeUserLastUpdated" => 'Admin',
                    "UOM_PACKINGMATERIALWEIGHT1" => $request->UOM_PACKINGMATERIALWEIGHT1,
                    "UOM_PACKINGMATERIALWEIGHT2" => $request->UOM_PACKINGMATERIALWEIGHT2,
                    "UOM_PACKINGMATERIALWEIGHT3" => $request->UOM_PACKINGMATERIALWEIGHT3,
                    "UOM_PACKINGMATERIALWEIGHT4" => $request->UOM_PACKINGMATERIALWEIGHT4,
                    "UOM_PACKINGMATERIALWEIGHT5" => $request->UOM_PACKINGMATERIALWEIGHT5,
                    "UOM_PACKINGMATERIALWEIGHT6" => $request->UOM_PACKINGMATERIALWEIGHT6,
                    "UOM_PACKINGMATERIALWEIGHT7" => $request->UOM_PACKINGMATERIALWEIGHT7,
                    "UOM_BAGSPERSTDPALLET"       => $request->UOM_BAGSPERSTDPALLET,
                    "UOM_IndDisplayOrderBy"       => $request->UOM_IndDisplayOrderBy,
                ];

                $category = UnitOfMeasure::where('UOM_IdUnit', $request->UOM_IdUnit)->update($data);

                DB::commit();
                $status = "1";
                $message = "Record updated successfully";
            } catch (\Exception $e) {
                DB::rollback();
                $message = "Transaction Faild: " . $e->getMessage();
            }
        }

        return response()->json(['status' => $status, 'errors' => (object)$errors, 'message' => $message, 'oData' => (object)$o_data], 200);
    }

    public function delete(REQUEST $request)
    {
        $status   = "0";
        $message  = "";
        $o_data   = [];
        $errors   = [];
        $validator = Validator::make($request->all(), [
            'access_token' => 'required',
            'UOM_IdUnit' => 'required'
        ]);

        if ($validator->fails()) {
            $status = "0";
            $message = "Validation error occured";
            $errors = $validator->messages();
        } else {
            $user = $this->validateAccesToken($request->access_token);
            $check = UnitOfMeasure::where(['UOM_IdUnit' => strtoupper($request->UOM_IdUnit)])->first();
            if ($check) {
                DB::beginTransaction();
                try {
                    // $check_vat_region = Ledger::where(['led_ledgergroup'=>$check->ACC_IdRecord,'led_compcode'=>$check->ACC_IdCompany])->get()->count();
                    // if($check_vat_region > 0){
                    //   $message = "You cant delete this record its associated with other record";
                    // }else{

                    UnitOfMeasure::where(['UOM_IdUnit' => strtoupper($request->UOM_IdUnit)])->delete();
                    DB::commit();
                    $status = "1";
                    $message = "Measure deleted successfully";

                    //}
                } catch (\Exception $e) {
                    DB::rollback();
                    $message = "Transaction Faild: " . $e->getMessage();
                }
            } else {
                $message = "record not found";
            }
        }

        return response()->json(['status' => $status, 'error' => (object)$errors, 'message' => $message, 'oData' => (object)$o_data], 200);
    }

    public function insertOrUpdate_CONT_CommodityUOM(Request $request)
    {
        $validator = Validator::make(
            $request->all(),
            []
            // [
            //     '*.CUM_IdUnit'        => 'required',
            //     '*.CUM_IdCommodity'   => 'required',
            //     '*.CUM_IdGrade'       => 'required',
            // ],
            // [
            //     '*.CUM_IdUnit.required'       => 'This field is required',
            //     '*.CUM_IdCommodity.required'  => 'This field is required',
            //     '*.CUM_IdGrade.required'      => 'This field is required',
            // ]
        );

        // Initialize variables
        $status = "success";
        $errors = [];
        $message = "";
        $o_data = [];

        try {
            if ($validator->fails()) {
                $status = "error";
                $message = "Validation error occurred";
                $errors = $validator->messages();
            } else {
                // Validation passed, create or update data
                $data = $request->all();
                foreach ($data as $item) {
                    // Find the record or create a new one
                    $check = CONT_CommodityUOM::where('CUM_IdUnit', $item['CUM_IdUnit'])
                        ->where('CUM_IdCommodity', $item['CUM_IdCommodity'])
                        ->where('CUM_IdGrade', $item['CUM_IdGrade'])->first();
                    if (!empty($check)) {
                        DB::table('CONT_CommodityUOM')->where('CUM_IdUnit', $item['CUM_IdUnit'])
                            ->where('CUM_IdCommodity', $item['CUM_IdCommodity'])
                            ->where('CUM_IdGrade',  $item['CUM_IdGrade'])->update([
                                'CUM_IdUnit'      => $item['CUM_IdUnit'],
                                'CUM_IdCommodity' => $item['CUM_IdCommodity'],
                                'CUM_IdGrade' =>        $item['CUM_IdGrade'],
                                'CUM_PACKINGMATERIALWEIGHT1'     => isset($item['CUM_PACKINGMATERIALWEIGHT1']) ? $item['CUM_PACKINGMATERIALWEIGHT1'] : null,
                                'CUM_PACKINGMATERIALWEIGHT2'     => isset($item['CUM_PACKINGMATERIALWEIGHT2']) ? $item['CUM_PACKINGMATERIALWEIGHT2'] : null,
                                'CUM_PACKINGMATERIALWEIGHT3'     => isset($item['CUM_PACKINGMATERIALWEIGHT3']) ? $item['CUM_PACKINGMATERIALWEIGHT3'] : null,
                                'CUM_PACKINGMATERIALWEIGHT4'     => isset($item['CUM_PACKINGMATERIALWEIGHT4']) ? $item['CUM_PACKINGMATERIALWEIGHT4'] : null,
                                'CUM_PACKINGMATERIALWEIGHT5'     => isset($item['CUM_PACKINGMATERIALWEIGHT5']) ? $item['CUM_PACKINGMATERIALWEIGHT5'] : null,
                                'CUM_PACKINGMATERIALWEIGHT6'     => isset($item['CUM_PACKINGMATERIALWEIGHT6']) ? $item['CUM_PACKINGMATERIALWEIGHT6'] : null,
                                'CUM_PACKINGMATERIALWEIGHT7'     => isset($item['CUM_PACKINGMATERIALWEIGHT7']) ? $item['CUM_PACKINGMATERIALWEIGHT7'] : null,
                                'CUM_PACKINGMATERIALWEIGHT8'     => isset($item['CUM_PACKINGMATERIALWEIGHT8']) ? $item['CUM_PACKINGMATERIALWEIGHT8'] : null,
                                'CUM_PACKINGMATERIALWEIGHT9'     => isset($item['CUM_PACKINGMATERIALWEIGHT9']) ? $item['CUM_PACKINGMATERIALWEIGHT9']  : null,
                                'CUM_BAGSPERSTDPALLET'           => isset($item['CUM_BAGSPERSTDPALLET']) ? $item['CUM_BAGSPERSTDPALLET'] : null,
                            ]);
                    } else {
                        DB::table('CONT_CommodityUOM')->insert(
                            [[
                                'CUM_IdUnit'      => $item['CUM_IdUnit'],
                                'CUM_IdCommodity' => $item['CUM_IdCommodity'],
                                'CUM_IdGrade' => $item['CUM_IdGrade'],
                                'CUM_PACKINGMATERIALWEIGHT1'     => isset($item['CUM_PACKINGMATERIALWEIGHT1']) ? $item['CUM_PACKINGMATERIALWEIGHT1'] : null,
                                'CUM_PACKINGMATERIALWEIGHT2'     => isset($item['CUM_PACKINGMATERIALWEIGHT2']) ? $item['CUM_PACKINGMATERIALWEIGHT2'] : null,
                                'CUM_PACKINGMATERIALWEIGHT3'     => isset($item['CUM_PACKINGMATERIALWEIGHT3']) ? $item['CUM_PACKINGMATERIALWEIGHT3'] : null,
                                'CUM_PACKINGMATERIALWEIGHT4'     => isset($item['CUM_PACKINGMATERIALWEIGHT4']) ? $item['CUM_PACKINGMATERIALWEIGHT4'] : null,
                                'CUM_PACKINGMATERIALWEIGHT5'     => isset($item['CUM_PACKINGMATERIALWEIGHT5']) ? $item['CUM_PACKINGMATERIALWEIGHT5'] : null,
                                'CUM_PACKINGMATERIALWEIGHT6'     => isset($item['CUM_PACKINGMATERIALWEIGHT6']) ? $item['CUM_PACKINGMATERIALWEIGHT6'] : null,
                                'CUM_PACKINGMATERIALWEIGHT7'     => isset($item['CUM_PACKINGMATERIALWEIGHT7']) ? $item['CUM_PACKINGMATERIALWEIGHT7'] : null,
                                'CUM_PACKINGMATERIALWEIGHT8'     => isset($item['CUM_PACKINGMATERIALWEIGHT8']) ? $item['CUM_PACKINGMATERIALWEIGHT8'] : null,
                                'CUM_PACKINGMATERIALWEIGHT9'     => isset($item['CUM_PACKINGMATERIALWEIGHT9']) ? $item['CUM_PACKINGMATERIALWEIGHT9']  : null,
                                'CUM_BAGSPERSTDPALLET'           => isset($item['CUM_BAGSPERSTDPALLET']) ? $item['CUM_BAGSPERSTDPALLET'] : null,
                                // Add other unique fields here as needed
                            ]]
                        );
                    }

                    $message = "Data saved successfully. ";
                }
            }
        } catch (\Exception $e) {
            $status = "error";
            $message = "An error occurred while processing the request.";
            $errors[] = $e->getMessage();
        }

        // Return the JSON response
        return response()->json([
            'status' => $status,
            'errors' => (object)$errors,
            'message' => $message,
            'oData' => (object)$o_data
        ], 200);
    }


    public function UOM_CommodityUOM(Request $request, $uom)
    {
        $status = 'error'; // Default status
        $errors = []; // Initialize errors array
        $message = ''; // Initialize message
        // dd(CONT_CommodityUOM::get());
        // Check if $uom is not empty
        if (!empty($uom)) {
            // Fetch CONT_CommodityUOM data based on $uom
            $CONT_CommodityUOM = CONT_CommodityUOM::where('CUM_IdUnit', $uom)->get();

            $status = 'success'; // Update status
            $message = 'Data retrieved successfully'; // Update message
            $responseData = $CONT_CommodityUOM; // Assign fetched data to responseData
        } else {
            // $uom is empty, set appropriate error message
            $errors[] = 'UOM parameter is empty';
        }

        // Return the JSON response
        return response()->json([
            'status' => $status,
            'error' => (object)$errors,
            'message' => $message,
            'oData' => $responseData
        ], 200);
    }



    public function system_parameter(REQUEST $request)
    {

        // Fetch data from CONT_CommodityUOM table
        $productSpecificComposition = CommodityCompanyRelation::Select('CCO_IdCommodity', 'CCO_IdGrade')->where('CCO_IdCompany', 'C02')->get();
        $UnitOfMeasure = UnitOfMeasure::pluck('UOM_IdUnit');
        // dd($UnitOfMeasure);
        // Fetch system parameters
        $systemParameters = SystemParameter::select([
            'SYP_NmeCode',
            'SYP_DscValue',
        ])
            ->where('SYP_NmeCode', 'like', 'PACKING-CONTENT-NAME-%')
            ->where('SYP_IdCompany', 'C02')
            ->where('SYP_DscValue', '!=', '')
            ->get();

        // Prepare the response data
        $status = 'success'; // Assuming the operation was successful
        $errors = []; // No errors in this case
        $message = 'Data retrieved successfully'; // A message indicating success

        // Combine both data sets into an associative array
        $responseData = [
            'productSpecificComposition' => $productSpecificComposition,
            'UnitOfMeasure' => $UnitOfMeasure,
            'systemParameters' => $systemParameters,
        ];
        // dd($systemParameters);
        // Return the JSON response
        return response()->json([
            'status' => $status,
            'error' => (object)$errors,
            'message' => $message,
            'oData' => (object)$responseData
        ], 200);
    }
}
